@@ -11,7 +11,7 @@ use Lemonldap::NG::Portal::Simple;
 use Lemonldap::NG::Portal::_SAML;
 our @ISA = qw(Lemonldap::NG::Portal::_SAML);
 
-our $VERSION = '1.9.8';
+our $VERSION = '1.9.9';
 
 ## @method void issuerDBInit()
 # Load and check SAML configuration
@@ -153,7 +153,7 @@ sub issuerForUnAuthUser {
             $self->lmLog( "$sp match $spConfKey SP in configuration", 'debug' );
 
             # Store values in %ENV
-            $ENV{"llng_saml_sp"} = $sp;
+            $ENV{"llng_saml_sp"}        = $sp;
             $ENV{"llng_saml_spconfkey"} = $spConfKey;
 
             # Do we check signature?
@@ -306,7 +306,7 @@ sub issuerForUnAuthUser {
             $self->lmLog( "$sp match $spConfKey SP in configuration", 'debug' );
 
             # Store values in %ENV
-            $ENV{"llng_saml_sp"} = $sp;
+            $ENV{"llng_saml_sp"}        = $sp;
             $ENV{"llng_saml_spconfkey"} = $spConfKey;
 
             # Do we check signature?
@@ -495,7 +495,7 @@ sub issuerForUnAuthUser {
             $self->lmLog( "$sp match $spConfKey SP in configuration", 'debug' );
 
             # Store values in %ENV
-            $ENV{"llng_saml_sp"} = $sp;
+            $ENV{"llng_saml_sp"}        = $sp;
             $ENV{"llng_saml_spconfkey"} = $spConfKey;
 
             # Do we check signature?
@@ -1357,6 +1357,18 @@ sub issuerForAuthUser {
                     'debug' );
             }
 
+            # Force AllowCreate to TRUE for transient/persistent NameIDPolicy
+            if ( $login->request()->NameIDPolicy ) {
+                my $nif = $login->request()->NameIDPolicy->Format();
+                if (   $nif eq $self->getNameIDFormat("transient")
+                    or $nif eq $self->getNameIDFormat("persistent") )
+                {
+                    $self->lmLog( "Force AllowCreate flag in NameIDPolicy",
+                        'debug' );
+                    eval { $login->request()->NameIDPolicy()->AllowCreate(1); };
+                }
+            }
+
             # Validate request
             unless ( $self->validateRequestMsg( $login, 1, 1 ) ) {
                 $self->lmLog( "Unable to validate SSO request message",
