@@ -16,7 +16,7 @@ use Crypt::OpenSSL::Bignum;
 use utf8;
 use base qw(Lemonldap::NG::Portal::_Browser);
 
-our $VERSION = '1.9.6';
+our $VERSION = '1.9.7';
 our $oidcCache;
 
 BEGIN {
@@ -389,7 +389,7 @@ sub buildHybridAuthnResponse {
 }
 
 ## @method String getAuthorizationCodeAccessToken(String op, String code, String auth_method)
-# Get Token response with autorization code
+# Get Token response with authorization code
 # @param op OpenIP Provider configuration key
 # @param code Code
 # @param auth_method Authentication Method
@@ -1047,6 +1047,34 @@ sub returnRedirectError {
     $self->quit;
 }
 
+## @method void returnJSONStatus(String content, int status_code);
+## Print JSON content
+## @param content Message
+## @param status_code The HTTP status code to return
+## @return void
+sub returnJSONStatus {
+    my ( $self, $content, $status_code ) = @_;
+
+    # We use to_json because values are already UTF-8 encoded
+    my $json = to_json( $content, { pretty => 1 } );
+
+    if ( $ENV{MOD_PERL} ) {
+        my $r = CGI->new->r;
+        $r->status($status_code);
+        $r->content_type("application/json; charset=UTF-8");
+        $r->rflush;
+        $r->status(200);
+    }
+    else {
+        print $self->header(
+            -type    => 'application/json',
+            -charset => 'UTF-8',
+            -status  => $status_code
+        );
+    }
+    print $json;
+}
+
 ## @method void returnJSONError(String error);
 # Print JSON error
 # @param error Error message
@@ -1054,14 +1082,7 @@ sub returnRedirectError {
 sub returnJSONError {
     my ( $self, $error ) = @_;
     my $content = { "error" => "$error" };
-
-    # We use to_json because values are already UTF-8 encoded
-    my $json = to_json( $content, { pretty => 1 } );
-
-# TODO Send 400 return code
-# CGI always add HTML code to non 200 return code, which is not compatible with JSON response
-    print $self->header('application/json');
-    print $json;
+    $self->returnJSONStatus( $content, 400 );
 }
 
 ## @method void returnJSON(String content);
@@ -1070,14 +1091,7 @@ sub returnJSONError {
 # @return void
 sub returnJSON {
     my ( $self, $content ) = @_;
-
-    # We use to_json because values are already UTF-8 encoded
-    my $json = to_json( $content, { pretty => 1 } );
-
-    $self->lmLog( "Return JSON: $json", 'debug' );
-
-    print $self->header('application/json');
-    print $json;
+    $self->returnJSONStatus( $content, 200 );
 }
 
 ## @method void returnBearerError(String error_code, String error_message);
@@ -1130,6 +1144,10 @@ sub getEndPointAccessToken {
         $self->lmLog( "Bearer access token", 'debug' );
         $access_token = $1;
     }
+    elsif ( $self->param('access_token') ) {
+        $self->lmLog( "GET/POST access token", 'debug' );
+        $access_token = $self->param('access_token');
+    }
 
     return $access_token;
 }
@@ -1374,8 +1392,8 @@ sub key2jwks {
     my $rsa_pub = Crypt::OpenSSL::RSA->new_public_key($key);
     my @params  = $rsa_pub->get_key_parameters();
 
-    $hash->{n} = encode_base64( $params[0]->to_bin(), "" );
-    $hash->{e} = encode_base64( $params[1]->to_bin(), "" );
+    $hash->{n} = encode_base64url( $params[0]->to_bin(), "" );
+    $hash->{e} = encode_base64url( $params[1]->to_bin(), "" );
 
     return $hash;
 }
@@ -1573,7 +1591,7 @@ Build Authentication Response URI for Hybrid Flow
 
 =head2 getAuthorizationCodeAccessToken
 
-Get Token response with autorization code
+Get Token response with authorization code
 
 =head2 checkTokenResponseValidity
 
@@ -1619,6 +1637,10 @@ Create Hash
 
 Create error redirection
 
+=head2 returnJSONStatus
+
+Print JSON content
+
 =head2 returnJSONError
 
 Print JSON error
